@@ -2,10 +2,9 @@ import pexpect
 
 from .Connection import Connection
 
-# Adjust the PTY window size to 800 characters wide to try to avoid truncating
-# command output
+# Increase the PTY window size to to try to avoid truncating command output
 PTY_WINSIZE_ROWS = 24
-PTY_WINSIZE_COLS = 800
+PTY_WINSIZE_COLS = 500
 
 SSH_PORT = 22
 
@@ -26,18 +25,22 @@ class Ssh(Connection):
 
         Connection.__init__(self)
 
-    def connect(self, logfile):
+    def connect(self, logfile, logger=None):
         """ Start the SSH connection.
-        @param logfile      Log file to save connection outputs.
+        @param logfile  Log file to save connection outputs.
+        @param logger   Optional logger for debug messages
         """
-        # TODO
-        #self.logger.debug("Connecting to ssh (%s).", self.ip)
-
+        if logger != None:
+            logger.debug("Connecting to SSH (%s).", self.ip)
         self.terminal = pexpect.spawn(
-            'ssh -p {2} {0}@{1}'.format(self.user, self.ip, self.port), logfile=logfile, encoding='utf-8')
+                'ssh -p {2} {0}@{1}'.format(self.user, self.ip, self.port),
+                logfile=logfile,
+                encoding='utf-8')
         self.terminal.setwinsize(PTY_WINSIZE_ROWS, PTY_WINSIZE_COLS)
 
-    def disconnect(self):
+    def disconnect(self, logger=None):
         """ For SSH, the connection is closed during the logout
+        @param logger   Optional logger for debug messages
         """
-        pass
+        if logger != None:
+            logger.debug("Disconnecting from SSH (%s).", self.ip)
